@@ -2,8 +2,13 @@
 from django.contrib.sitemaps import Sitemap
 from django.db.models import get_models
 
+from mezzanine.conf import settings
 from mezzanine.core.models import Displayable
 
+blog_installed = "mezzanine.blog" in settings.INSTALLED_APPS
+if blog_installed:
+    from mezzanine.blog.models import BlogPost
+
 
 class DisplayableSitemap(Sitemap):
     """
@@ -16,8 +21,13 @@ class DisplayableSitemap(Sitemap):
         Return all published items for models that subclass
         ``Displayable``.
         """
-        items = []
+        items = {}
         for model in get_models():
             if issubclass(model, Displayable):
-                items.extend(model.objects.published())
-        return items
+                for item in model.objects.published():
+                    items[item.get_absolute_url()] = item
+        return items.values()
+
+    def lastmod(self, obj):
+        if blog_installed and isinstance(obj, BlogPost):
+            return obj.publish_date
