@@ -20,5 +20,5 @@ urlpatterns += patterns("mezzanine.core.views",
     url("^search/$", "search", name="search"),
     url("^set_site/$", "set_site", name="set_site"),
     url("^set_device/(?P<device>.*)/$", "set_device", name="set_device"),
-    url("^static_proxy/$", "static_proxy", name="static_proxy"),
+    url("^asset_proxy/$", "static_proxy", name="static_proxy"),
 )
