@@ -3,10 +3,25 @@ from django.db import models
 from django.utils.translation import ugettext_lazy as _
 
 from mezzanine.core.models import Displayable, Orderable, RichText
-from mezzanine.utils.urls import admin_url, slugify
+from mezzanine.pages.fields import MenusField
+from mezzanine.pages.managers import PageManager
+from mezzanine.utils.urls import admin_url, path_to_slug, slugify
 
 
-class Page(Orderable, Displayable):
+class BasePage(Orderable, Displayable):
+    """
+    Exists solely to store ``PageManager`` as the main manager.
+    If it's defined on ``Page``, a concrete model, then each
+    ``Page`` subclass loses the custom manager.
+    """
+
+    objects = PageManager()
+
+    class Meta:
+        abstract = True
+
+
+class Page(BasePage):
     """
     A page in the page tree. This is the base class that custom content types
     need to subclass.
@@ -14,8 +29,7 @@ class Page(Orderable, Displayable):
 
     parent = models.ForeignKey("Page", blank=True, null=True,
         related_name="children")
-    in_navigation = models.BooleanField(_("Show in navigation"), default=True)
-    in_footer = models.BooleanField(_("Show in footer"))
+    in_menus = MenusField(_("Show in menus"), blank=True, null=True)
     titles = models.CharField(editable=False, max_length=1000, null=True)
     content_model = models.CharField(editable=False, max_length=50, null=True)
     login_required = models.BooleanField(_("Login required"),
@@ -52,8 +66,8 @@ class Page(Orderable, Displayable):
 
     def save(self, *args, **kwargs):
         """
-        Create the titles field using the titles up the parent chain and set
-        the initial value for ordering.
+        Create the titles field using the titles up the parent chain
+        and set the initial value for ordering.
         """
         if self.id is None:
             self.content_model = self._meta.object_name.lower()
@@ -65,6 +79,37 @@ class Page(Orderable, Displayable):
         self.titles = " / ".join(titles)
         super(Page, self).save(*args, **kwargs)
 
+    def get_ascendants(self, for_user=None):
+        """
+        Returns the ascendants for the page. Ascendants are cached in
+        the ``_ascendants`` attribute, which is populated when the page
+        is loaded via ``Page.objects.with_ascendants_for_slug``.
+        """
+        if not self.parent_id:
+            # No parents at all, bail out.
+            return []
+        if not hasattr(self, "_ascendants"):
+            # _ascendants has not been either page.get_ascendants or
+            # Page.objects.assigned by with_ascendants_for_slug, so
+            # run it to see if we can retrieve all parents in a single
+            # query, which will occur if the slugs for each of the pages
+            # have not been customised.
+            if self.slug:
+                args = (self.slug, for_user)
+                pages = Page.objects.with_ascendants_for_slug(**args)
+                self._ascendants = pages[0]._ascendants
+            else:
+                self._ascendants = []
+        if not self._ascendants:
+            # Page has a parent but with_ascendants_for_slug failed to
+            # find them due to custom slugs, so retrieve the parents
+            # recursively.
+            child = self
+            while child.parent_id is not None:
+                self._ascendants.append(child.parent)
+                child = child.parent
+        return self._ascendants
+
     @classmethod
     def get_content_models(cls):
         """
@@ -114,7 +159,7 @@ class Page(Orderable, Displayable):
         """
         Dynamic ``add`` permission for content types to override.
         """
-        return True
+        return self.slug != "/"
 
     def can_change(self, request):
         """
@@ -138,19 +183,18 @@ class Page(Orderable, Displayable):
         current_page_id = getattr(current_page, "id", None)
         current_parent_id = getattr(current_page, "parent_id", None)
         # Am I a child of the current page?
-        self.is_child = self.parent_id == current_page_id
+        self.is_current_child = self.parent_id == current_page_id
+        self.is_child = self.is_current_child  # Backward compatibility
         # Is my parent the same as the current page's?
         self.is_current_sibling = self.parent_id == current_parent_id
         # Am I the current page?
-        from mezzanine.urls import PAGES_SLUG
         try:
             request = context["request"]
         except KeyError:
             # No request context, most likely when tests are run.
             self.is_current = False
         else:
-            slug = request.path.strip("/").replace(PAGES_SLUG, "", 1)
-            self.is_current = self.slug == slug
+            self.is_current = self.slug == path_to_slug(request.path_info)
 
         # Is the current page me or any page up the parent chain?
         def is_c_or_a(page_id):
