@@ -27,8 +27,14 @@ class Library(template.Library):
             class AsTagNode(template.Node):
                 def render(self, context):
                     parts = token.split_contents()
+
                     # Resolve variables if their names are given.
-                    args = [context.get(arg, arg) for arg in parts[1:-2]]
+                    def resolve(arg):
+                        try:
+                            return template.Variable(arg).resolve(context)
+                        except template.VariableDoesNotExist:
+                            return arg
+                    args = [resolve(arg) for arg in parts[1:-2]]
                     context[parts[-1]] = tag_func(*args)
                     return ""
             return AsTagNode()
