@@ -21,6 +21,18 @@ PYFLAKES_IGNORE = (
     "redefinition of unused 'Feed'",
     "redefinition of unused 'feed'",
 
+    # Backward compatibility for timezone support
+    "redefinition of unused 'now'",
+
+    # No caching fallback
+    "redefinition of function 'nevercache'",
+
+    # Dummy fallback in templates for django-compressor
+    "redefinition of function 'compress'",
+
+    # Fabic config fallback
+    "redefinition of unused 'conf'",
+
 )
 
 
